module BurrowsWheeler
  # > CircularString.new(str, 5).to_s
  # => "ADABRA!ABRAC",

  class CircularString
    include Comparable

    attr_reader :string
    attr_reader :shift

    def initialize(string, shift)
      @string = string
      @shift = shift
    end

    def [](index)
      return nil if index < 0 || index >= string.length
      idx = (index + @shift) % @string.length
      @string[idx]
    end

    def last
      self[length - 1]
    end

    def subsequence(start, finish)
      len = finish - start
      CircularString.new(@string[start, len], @shift)
    end

    def <=>(other)
      len = [length, other.length].min

      (0..len - 1).each do |i|
        c1, c2 = self[i], other[i]
        return (c1 <=> c2) if c1 != c2
      end

      length <=> other.length
    end

    def length
      @string.length
    end

    def to_s
      (0..@string.length - 1).reduce('') do |str, idx|
        str + self[idx]
      end
    end
  end
end
